//************************************************************************
// KS108.H
// LowLevel Routinen fr LCD-Displays mit KS0108
//
// Hier fr Displaytech 64240A
//
// hk@holger-klabunde.de
// www.holger-klabunde.de
// 17.11.2006
// Compiler MCC18 V2.3
//************************************************************************

#ifndef KS108_H
#define KS108_H

#define NOP() _asm nop _endasm

//#################################################
//# Number of NOP's below is based on 12MHz Clock #
//#################################################

// Only define ONE of these !!
#define LCD_WIDTH_240         //Display Width for 240x64
//#define LCD_WIDTH_128         //Display Width for 128x64

// KS108 displays have one or more ks108 controllers.
// 240x64 has 4 controllers, 128x64 has 2 controllers
#ifdef LCD_WIDTH_240
 #define LCD_WIDTH		240
 #define MAX_CONTROLLERS	4
#endif

#ifdef LCD_WIDTH_128
 #define LCD_WIDTH		128
 #define MAX_CONTROLLERS	2
#endif

#define LCD_HEIGHT	64          //Display Height
#define BYTES_PER_ROW   LCD_WIDTH/8 //Bytes per row
#define FONT_PITCH	8

//PORTD is DATA
//PORTB is CTRL
#define INIT_CONTROL_PINS() { TRISD=0xFF; TRISB=0x00; RSTB_ON(); } //control pins to output

#define DATA_PORT	PORTD
#define DATA_DIR_IN()  	{ TRISD=0xFF; DATA_PORT=0xFF; } // set io-pins to inputs with pullups
#define DATA_DIR_OUT()	TRISD=0x00; // set io-pins to outputs
#define READ_DATA()   	PORTD	  // read PIN, ! NOT ! PORT
#define WRITE_DATA(a)  	DATA_PORT=(a); // write to data port

#define RS_ON()	       	{ PORTBbits.RB7=1; }
#define RS_OFF()       	{ PORTBbits.RB7=0; }

#define RW_ON()	       	{ PORTBbits.RB6=1; }
#define RW_OFF()       	{ PORTBbits.RB6=0; } //min 140ns

#define E_ON()	       	{ PORTBbits.RB5=1; Wait_for_E_High(); } //min 450ns
#define E_OFF()       	{ PORTBbits.RB5=0; Wait_for_E_Low(); } //min 450ns
//#define E_OFF()       	{ PORTBbits.RB5=0; CheckBusy(); } //min 450ns

#define RSTB_ON()      	PORTBbits.RB4=1;
#define RSTB_OFF()     	PORTBbits.RB4=0;

#define CS1_ON()      	PORTBbits.RB0=1;
#define CS1_OFF()     	PORTBbits.RB0=0;

#define CS2_ON()      	PORTBbits.RB1=1;
#define CS2_OFF()     	PORTBbits.RB1=0;

#define CS3_ON()      	PORTBbits.RB2=1;
#define CS3_OFF()     	PORTBbits.RB2=0;

#define CS4_ON()      	PORTBbits.RB3=1;
#define CS4_OFF()     	PORTBbits.RB3=0;

//##########################################################
//Ein Byte in das Display schreiben
//Register Select mu vorher entsprechend gesetzt werden !
//Warning[2058] F:\DATEN\PicC18\USB_KS108\CDC18F4550_KS108\user.c 131 : call of function without prototype
/*
#define WriteDisplay(a) { \
 			RW_OFF(); \
 			NOP(); \
 			DATA_DIR_OUT(); \
 			WRITE_DATA((a)); \
 			E_ON(); \
 			E_OFF(); \
};
*/

//Alle CS Leitungen gleichzeitig bearbeiten
#define SetChipNumber(a) { PORTB&=0xF0; NOP(); PORTB|=(1<<(a)); }

//##########################################################
//Pagenummer des Chips setzen
#define SetPageNumber(a) { \
			 RS_OFF(); \
			 WriteDisplay(0xB8|(a)); \
                         RS_ON(); \
};

//##########################################################
//Spaltennummer (X-Position) des Chips setzen
#define SetRow(a) { \
                   RS_OFF(); \
                   WriteDisplay(0x40| ((a)%64)); \
                   RS_ON(); \
};

//##########################################################
//Gewnschte Koordinate auf dem Display einstellen
//Warning[2058] F:\DATEN\PicC18\USB_KS108\CDC18F4550_KS108\user.c 131 : call of function without prototype
/*
#define SetPosition(x,y) { \
                    SetChipNumber((x)/64); \
                    SetPageNumber((y)/8); \
                    SetRow((x)); \
};
*/

extern unsigned char ReadDisplay(void);
extern void WriteDisplay(unsigned char dat);
extern void SetPixel(unsigned char xpos, unsigned char ypos, unsigned char mode);
//extern unsigned char GetPixel(unsigned char xpos, unsigned char ypos);
//extern unsigned char SetPageNumber(unsigned char num);
//extern unsigned char SetRow(unsigned char xpos);
extern void SetPosition(unsigned char xpos, unsigned char ypos);
extern void InitDisplay(void);
extern void ClearScreen(void);
extern void CheckBusy(void);
extern void LCDChar(unsigned char ch);
extern void LCDPos(unsigned char row, unsigned char col);
extern void MakeCGRam(void);
extern void LCDCGChar(unsigned char ch, unsigned char xpos, unsigned char ypos);

extern unsigned char xcursor,ycursor;
extern unsigned char cgram[9];
extern unsigned char xpos,ypos;
extern unsigned char pixbuffer[];
extern unsigned char pixlen;

#endif
