/*********************************************************************
 *
 *                Microchip USB C18 Firmware Version 1.0
 *
 *********************************************************************
 * FileName:        user.h
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18
 * Compiler:        C18 2.30.01+
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Rawin Rojvanit       11/19/04    Original.
 * Holger Klabunde      02/09/06    Modified.
 ********************************************************************/

#ifndef USER_H
#define USER_H

#include "ks108.h"
#include "fontsoft.h"
#include "delay.h"

#define BUFFER_SIZE	64

//Display commands
#define CLEAR_SCREEN		0xB0
#define SET_CURSOR		0xB1
#define CURSOR_HOME		0xB2
#define SET_CONTRAST		0xB3 //0..255, 0=off
#define SET_LIGHT		0xB4 //0..255, 0=off
#define SET_CGCHAR		0xB5 //0..31
//                              0xB6 //free
#define SET_GFX			0xB7 //Set pixel array, 6 or 8 pixel at once
#define CLEAR_TEXT		0xB8 //Clear text screen, not used
#define CLEAR_GFX		0xB9 //Clear graphics screen, not used
#define SET_GFXBUFFER		0xBA //Set buffered pixel arrays, 6 or 8 pixel at once
#define SET_BITMAP		0xBB //Receive a bitmap. Each row can contain up to 255 bytes
#define SET_OUTPUTS		0xBE //Receive a portnumber and a value for the port to set io pins
#define LAST_COMMAND		0xBF

// State declarations for ProcessIo() states in user.c
#define NO_STATE		0
#define STATE_COMMAND		1
#define STATE_GETCOL		2
#define STATE_GETROW		3
#define STATE_GETCONTRAST	4
#define STATE_GETBACKLIGHT	5
#define STATE_GETCGADR		6
#define STATE_GETCG0		7
#define STATE_GETCG1		8
#define STATE_GETCG2		9
#define STATE_GETCG3		10
#define STATE_GETCG4		11
#define STATE_GETCG5		12
#define STATE_GETCG6		13
#define STATE_GETCG7		14
/*
#define STATE_GETPIXEL_X	15 //0..255
#define STATE_GETPIXEL_Y	16 //0..255
#define STATE_GETPIXEL_MODE	17
*/
#define STATE_GETGFX_X		18 //0..255
#define STATE_GETGFX_Y		19 //0..255
#define STATE_GETGFX_BYTE	20

#define STATE_GETGFXBUFFER_X	21 //0..255
#define STATE_GETGFXBUFFER_Y    22 //0..255
#define STATE_GETGFXBUFFER_LEN	23 //0..255
#define STATE_GFXBUFFER_FILL	24 //keep this state until buffer is filled

#define STATE_GETBITMAP_X	25 //0..255 X start position for the rows
#define STATE_GETBITMAP_Y	26 //0..255 Y position of the first row 
#define STATE_GETBITMAP_WIDTH	27 //in bytes  0..255
#define STATE_GETBITMAP_HEIGHT	28 //in pixels 0..255
#define STATE_GETBITMAP_GETROW	29 //get data for this row
#define STATE_GETBITMAP_FULLSCREEN	30 //get a whole screen bitmap !

#define STATE_GETOUTPUT_PORT	31
#define STATE_GETOUTPUT_VALUE	32

/** P U B L I C  P R O T O T Y P E S *****************************************/
void UserInit(void);
void ProcessIO(void);

#endif //USER_H
